# coding=utf-8

"""Integer Parameter."""
from generic_parameter import GenericParameter
from parameter_exceptions import (
    InvalidMaximumError, InvalidMinimumError, ValueOutOfBounds)


class NumericParameter(GenericParameter):
    """A subclass of generic parameter that accepts float or integer only.

    Known issue : You should set the minimum and the maximum value explicitly.
    See https://github.com/inasafe/inasafe/issues/2468#issuecomment-171124507

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(NumericParameter, self).__init__(guid)
        self._minimum_allowed_value = None
        self._maximum_allowed_value = None
        self._unit = None
        self._allowed_units = []

    @property
    def minimum_allowed_value(self):
        """Property for the minimum allowed value for the parameter.

        :returns: The minimum allowed value.
        :rtype: float, int
        """
        return self._minimum_allowed_value

    @minimum_allowed_value.setter
    def minimum_allowed_value(self, value):
        """Setter for the minimum allowed value for the parameter.

        :param value: The minimum allowed value.
        :type value: float, int

        :raises: InvalidMinimumError, TypeError
        """
        self._check_type(value)
        # If maximum is not set, we can set minimum regardless
        if self._maximum_allowed_value is None:
            self._minimum_allowed_value = value
            return
        # Otherwise it must be less than maximum
        if value <= self._maximum_allowed_value:
            self._minimum_allowed_value = value
            return

        raise InvalidMinimumError('Minimum must be less than maximum')

    @property
    def maximum_allowed_value(self):
        """Property for the maximum allowed value for the parameter.

        :returns: The maximum allowed value.
        :rtype: float, int
        """
        return self._maximum_allowed_value

    @maximum_allowed_value.setter
    def maximum_allowed_value(self, value):
        """Setter for the maximum allowed value for the parameter.

        :param value: The maximum allowed value.
        :type value: float, int

        :raises: InvalidMaximumError, TypeError
        """
        self._check_type(value)
        # If minimum is not set, we can set maximum regardless
        if self._minimum_allowed_value is None:
            self._maximum_allowed_value = value
            return
        # Otherwise it must be more than the minimum
        if value >= self._minimum_allowed_value:
            self._maximum_allowed_value = value
            return

        raise InvalidMaximumError('Maximum must be greater than minimum')

    @property
    def unit(self):
        """Property for the unit for the parameter.

        :returns: The unit of the parameter.
        :rtype: Unit

        """
        return self._unit

    @unit.setter
    def unit(self, unit):
        """Setter for unit for the parameter.

        :param unit: Unit for parameter
        :type unit: Unit

        """
        self._unit = unit

    @property
    def allowed_units(self):
        """Property for the allowed_units for the parameter.

        :returns: The list of allowed unit of the parameter.
        :rtype: list

        """
        return self._allowed_units

    @allowed_units.setter
    def allowed_units(self, allowed_units):
        """Setter for allowed_units for the parameter.

        :param allowed_units: The list of allowed unit of the parameter.
        :type allowed_units: list

        """
        self._allowed_units = allowed_units

    @GenericParameter.value.setter
    def value(self, value):
        """Define the current _value for the parameter.

        .. note:: Overloads the setting in GenericParameter

        :param value: The _value of the parameter itself.
        :type value: str, bool, integer, float, list, dict

        :raises: TypeError
        """
        self._check_type(value)
        if self._minimum_allowed_value <= value <= self._maximum_allowed_value:
            self._value = value
        else:
            raise ValueOutOfBounds(
                'Value (%s) must be greater than %s and less than %s' % (
                    value,
                    self._minimum_allowed_value,
                    self._maximum_allowed_value))

    def serialize(self):
        """Convert the parameter into a dictionary.

        :return: The parameter dictionary.
        :rtype: dict
        """
        pickle = super(NumericParameter, self).serialize()
        pickle['minimum_allowed_value'] = self.minimum_allowed_value
        pickle['maximum_allowed_value'] = self.maximum_allowed_value
        pickle['unit'] = self.unit
        pickle['allowed_units'] = [unit.name for unit in self.allowed_units]
        return pickle



# coding=utf-8

"""Integer Parameter."""
import sys

from numeric_parameter import NumericParameter


class IntegerParameter(NumericParameter):
    """A subclass of numeric parameter that accepts integer only.

    .. note:: By default the min and max allowed values will be
        the platform specific largest and smallest int numbers.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(IntegerParameter, self).__init__(guid)
        self.expected_type = int
        self._minimum_allowed_value = - sys.maxint - 1
        self._maximum_allowed_value = sys.maxint
        self._unit = ''



# coding=utf-8
"""Test class for qt4_parameter_factory."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'test_qt4_parameter_factory'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'

import unittest

from PyQt4.QtGui import QApplication
from parameters.boolean_parameter import BooleanParameter
from parameters.float_parameter import FloatParameter
from parameters.qt_widgets.boolean_parameter_widget import BooleanParameterWidget
from parameters.qt_widgets.float_parameter_widget import FloatParameterWidget
from parameters.qt_widgets.test.custom_parameter.point_parameter import PointParameter
from parameters.qt_widgets.test.custom_parameter.point_parameter_widget import (
    PointParameterWidget)

from parameters.qt_widgets.qt4_parameter_factory import Qt4ParameterFactory


class TestQt4ParameterFactory(unittest.TestCase):
    application = QApplication([])

    def setUp(self):
        """SetUp for unit test."""
        self.boolean_parameter = BooleanParameter('1231231')
        self.boolean_parameter.name = 'Boolean'
        self.boolean_parameter.help_text = 'A boolean parameter'
        self.boolean_parameter.description = 'A test _description'
        self.boolean_parameter.is_required = True
        self.boolean_parameter.value = True

        self.float_parameter = FloatParameter()
        self.float_parameter.name = 'Float Parameter'
        self.float_parameter.is_required = True
        self.float_parameter.precision = 3
        self.float_parameter.minimum_allowed_value = 1.0
        self.float_parameter.maximum_allowed_value = 2.0
        self.float_parameter.help_text = 'Short help.'
        self.float_parameter.description = 'Long description for parameter.'
        self.float_parameter.unit = 'metres'
        self.float_parameter.value = 1.1

        self.point_parameter = PointParameter()
        self.point_parameter.name = 'Point Parameter'
        self.point_parameter.is_required = True
        self.point_parameter.help_text = 'Short help.'
        self.point_parameter.description = 'Long description for parameter.'
        self.point_parameter.value = (0, 1)

    def test_init(self):
        """Test initialize qt4 parameter factory."""
        parameters = [self.boolean_parameter, self.float_parameter]

        qt4_parameter_factory = Qt4ParameterFactory()
        widgets = []
        widget_classes = []

        for parameter in parameters:
            widget = qt4_parameter_factory.get_widget(parameter)
            widgets.append(widget)
            widget_classes.append(widget.__class__)

        expected_classes = [BooleanParameterWidget, FloatParameterWidget]
        message = 'Expected %s got %s' % (expected_classes, widget_classes)
        self.assertListEqual(widget_classes, expected_classes, message)

    def test_custom_parameter(self):
        """Test adding new custom parameter to the factory"""
        qt4_parameter_factory = Qt4ParameterFactory()
        qt4_parameter_factory.register_widget(
            PointParameter, PointParameterWidget)

        parameters = [
            self.boolean_parameter, self.float_parameter, self.point_parameter]

        widgets = []
        widget_classes = []

        for parameter in parameters:
            widget = qt4_parameter_factory.get_widget(parameter)
            widgets.append(widget)
            widget_classes.append(widget.__class__)

        expected_classes = [
            BooleanParameterWidget, FloatParameterWidget, PointParameterWidget]
        message = 'Expected %s got %s' % (expected_classes, widget_classes)
        self.assertListEqual(widget_classes, expected_classes, message)

if __name__ == '__main__':
    unittest.main()



# coding=utf-8
"""Numeric Parameter Widget"""

# noinspection PyPackageRequirements
from PyQt4.QtCore import Qt
from PyQt4.QtGui import QLabel, QSizePolicy, QWidget, QComboBox

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'numeric_parameter_widget'
__date__ = '8/21/14'
__copyright__ = 'ismail@kartoza.com'


class NumericParameterWidget(GenericParameterWidget):
    """Widget class for Numeric parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A NumericParameter object.
        :type parameter: NumericParameter

        """
        super(NumericParameterWidget, self).__init__(parameter, parent)

        self._input = QWidget()

        self._unit_widget = QLabel()
        self.set_unit()

        # Size policy
        self._spin_box_size_policy = QSizePolicy(
            QSizePolicy.Fixed, QSizePolicy.Fixed)

        label_policy = QSizePolicy(
            QSizePolicy.Minimum, QSizePolicy.Fixed)
        self._unit_widget.setSizePolicy(label_policy)

        # Add unit description to description
        description = self.help_label.text()
        if self._parameter.allowed_units:
            description += '<br><br>Available units:'
            description += '<ul>'
            for allowed_unit in self._parameter.allowed_units:
                description += '<li>'
                description += '<b>' + allowed_unit.name + '</b>: '
                description += allowed_unit.description or ''
                description += '</li>'
            description += '</ul>'
        self.help_label.setText(description)

    def get_parameter(self):
        """Obtain numeric parameter object from the current widget state.

        :returns: A NumericParameter from the current state of widget

        """
        self._parameter.value = self._input.value()
        if len(self._parameter.allowed_units) > 1:
            current_index = self._unit_widget.currentIndex()
            unit = self._unit_widget.itemData(current_index, Qt.UserRole)
            if hasattr(unit, 'toPyObject'):
                unit = unit.toPyObject()
            self._parameter.unit = unit
        return self._parameter

    def set_unit(self):
        """Set up label or combo box for unit."""
        if len(self._parameter.allowed_units) == 1:
            self._unit_widget = QLabel(self._parameter.unit.name)
            self._unit_widget.setToolTip(self._parameter.unit.help_text)
        elif len(self._parameter.allowed_units) > 1:
            self._unit_widget = QComboBox()
            index = -1
            current_index = -1
            for allowed_unit in self._parameter.allowed_units:
                name = allowed_unit.name
                tooltip = allowed_unit.help_text
                index += 1
                if allowed_unit.guid == self._parameter.unit.guid:
                    current_index = index
                self._unit_widget.addItem(name)
                self._unit_widget.setItemData(index, tooltip, Qt.ToolTipRole)
                self._unit_widget.setItemData(index, allowed_unit, Qt.UserRole)
            self._unit_widget.setCurrentIndex(current_index)
            self._unit_widget.setToolTip('Select your preferred unit')
            self._unit_widget.currentIndex()

    def set_value(self, value):
        """Set the value of the input

        :param value: The new value
        :type value: int, float
        """
        self._input.setValue(value)



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'metadata.py'
__date__ = '8/25/14'
__copyright__ = 'ismail@kartoza.com'
# constants
small_number = 2 ** -53  # I think this is small enough

# units
unit_generic = {
    'id': 'generic',
    'name': 'generic',
    'help_text': 'Generic units',
    'description': (
        '<b>Generic</b> units in an unspecified measurement system.'),
    'constraint': 'continuous',
    'default_attribute': 'units'  # applies to vector only
}
unit_litres = {
    'id': 'litres',
    'name': 'litres',
    'help_text': 'Litres are a metric unit of measure.',
    'description': (
        '<b>Litres</b> are an metric unit of measure. There are 1000 '
        'millilitres in 1 litres.'),
    'constraint': 'continuous',
    'default_attribute': 'litres'  # applies to vector only
}
unit_feet_depth = {
    'id': 'feet_depth',
    'name': 'feet',
    'help_text': 'Feet are an imperial unit of measure.',
    'description': (
        '<b>Feet</b> are an imperial unit of measure. There are 12 '
        'inches in 1 foot and 3 feet in 1 yard. '
        'In this case <b>feet</b> are used to describe the water depth.'),
    'constraint': 'continuous',
    'default_attribute': 'depth'  # applies to vector only
}
unit_metres_depth = {
    'id': 'metres_depth',
    'name': 'metres',
    'help_text': 'Metres are an metric unit of measure.',
    'description': (
        '<b>metres</b> are a metric unit of measure. There are 100 '
        'centimetres in 1 metre. In this case <b>metres</b> are used to '
        'describe the water depth.'),
    'constraint': 'continuous',
    'default_attribute': 'depth'  # applies to vector only
}
unit_mmi = {
    'id': 'mmi',
    'name': 'MMI',
    'help_text': 'MMI describe the intensity of ground shaking.',
    'description': (
        'The <b>Modified Mercalli Intensity (MMI)</b> scale describes '
        'the intensity of ground shaking from a earthquake based on the '
        'effects observed by people at the surface.'),
    'constraint': 'continuous',
    'default_attribute': 'mmi'  # applies to vector only
}



# coding=utf-8

"""Simple example of using parameters to define a parameter - how meta!"""

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'main'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'

import sys

from PyQt4.QtGui import QApplication, QWidget, QGridLayout
from float_parameter import FloatParameter
from string_parameter import StringParameter

from parameters.qt_widgets.parameter_container import ParameterContainer


def main():
    """Main function"""
    application = QApplication([])

    name_parameter = StringParameter('UUID-1')
    name_parameter.name = 'Resource name'
    name_parameter.help_text = (
        'Name of the resource that will be provided as part of minimum needs.'
        'e.g. Tea, Water etc.')
    name_parameter.description = (
        'A <b>resource</b> is something that you provide to displaced persons '
        'in the event of a disaster. The resource will be made available '
        'at IDP camps and may need to be stockpiled by contingency planners '
        'in their preparations for a disaster.')
    name_parameter.is_required = True
    name_parameter.value = ''

    description_parameter = StringParameter('UUID-1')
    description_parameter.name = 'Resource description'
    description_parameter.help_text = (
        'Description of the resource that will be provided as part of minimum '
        'needs. e.g. Tea, Water etc.')
    description_parameter.description = (
        'Description of the resource that will be provided as part of minimum '
        'needs. e.g. Tea, Water etc.')
    description_parameter.is_required = True
    description_parameter.value = ''

    unit_parameter = StringParameter('UUID-2')
    unit_parameter.name = 'Units'
    unit_parameter.help_text = (
        'Unit for the resources. e.g. litres, kg etc.')
    unit_parameter.description = (
        'A <b>unit</b> the basic measurement unit used for computing the '
        'allowance per individual. For example when planning water rations '
        'the units would be litres.')
    unit_parameter.is_required = True
    unit_parameter.value = ''

    minimum_parameter = FloatParameter('UUID-3')
    minimum_parameter.name = 'Minimum allowed'
    minimum_parameter.is_required = True
    minimum_parameter.precision = 3
    minimum_parameter.minimum_allowed_value = -99999.0
    minimum_parameter.maximum_allowed_value = 99999.0
    minimum_parameter.help_text = 'The minimum allowable quantity per person. '
    minimum_parameter.description = (
        'The <b>minimum</b> is the minimum allowed quantity of the resource '
        'per person. For example you may dictate that the water ration per '
        'person per day should never be allowed to be less than 0.5l.')
    minimum_parameter.value = 1.0

    maximum_parameter = FloatParameter('UUID-3')
    maximum_parameter.name = 'Minimum allowed'
    maximum_parameter.is_required = True
    maximum_parameter.precision = 3
    maximum_parameter.minimum_allowed_value = -99999.0
    maximum_parameter.maximum_allowed_value = 99999.0
    maximum_parameter.help_text = 'The maximum allowable quantity per person. '
    maximum_parameter.description = (
        'The <b>maximum</b> is the maximum allowed quantity of the resource '
        'per person. For example you may dictate that the water ration per '
        'person per day should never be allowed to be more than 50l.')
    maximum_parameter.value = 1.0

    maximum_parameter = FloatParameter('UUID-4')
    maximum_parameter.name = 'Minimum allowed'
    maximum_parameter.is_required = True
    maximum_parameter.precision = 3
    maximum_parameter.minimum_allowed_value = -99999.0
    maximum_parameter.maximum_allowed_value = 99999.0
    maximum_parameter.help_text = 'The maximum allowable quantity per person. '
    maximum_parameter.description = (
        'The <b>maximum</b> is the maximum allowed quantity of the resource '
        'per person. For example you may dictate that the water ration per '
        'person per day should never be allowed to be more than 50l.')
    maximum_parameter.value = 1.0

    default_parameter = FloatParameter('UUID-5')
    default_parameter.name = 'Default'
    default_parameter.is_required = True
    default_parameter.precision = 3
    default_parameter.minimum_allowed_value = -99999.0
    default_parameter.default_allowed_value = 99999.0
    default_parameter.help_text = 'The default allowable quantity per person. '
    default_parameter.description = (
        'The <b>default</b> is the default allowed quantity of the resource '
        'per person. For example you may indicate that the water ration per '
        'person per day should be 25l.')
    default_parameter.value = 1.0

    parameters = [
        name_parameter,
        description_parameter,
        unit_parameter,
        minimum_parameter,
        maximum_parameter,
        default_parameter]
    parameter_container = ParameterContainer(parameters)

    widget = QWidget()
    layout = QGridLayout()
    layout.addWidget(parameter_container)
    widget.setLayout(layout)
    widget.setGeometry(0, 0, 500, 500)

    widget.show()

    new_parameters = parameter_container.get_parameters()
    for new_parameter in new_parameters:
        print new_parameter.name, new_parameter.value

    sys.exit(application.exec_())

if __name__ == '__main__':
    main()



# coding=utf-8
"""Tests for float parameter."""

from unittest import TestCase

from parameters.float_parameter import FloatParameter

from parameters.parameter_exceptions import (
    InvalidMaximumError, InvalidMinimumError, ValueOutOfBounds)


class TestFloatParameter(TestCase):
    """Test class for FloatParameter."""
    def test_all(self):
        """Basic test of all properties."""
        parameter = FloatParameter()
        parameter.is_required = True
        parameter.minimum_allowed_value = 1.0
        parameter.maximum_allowed_value = 2.0
        parameter.value = 1.123

        self.assertEqual(1.123, parameter.value)

        with self.assertRaises(TypeError):
            parameter.value = 'Test'

        with self.assertRaises(ValueOutOfBounds):
            parameter.value = 3.0

        with self.assertRaises(ValueOutOfBounds):
            parameter.value = 0.5

    def test_set_minimum_allowed_value(self):
        """Test setter for minimum allowed value."""
        parameter = FloatParameter()

        with self.assertRaises(TypeError):
            parameter.maximum_allowed_value = 'One thousand'

        parameter.minimum_allowed_value = 1

        parameter.minimum_allowed_value = 1.0
        # Also check that it raises an error if it exceeds max
        parameter.maximum_allowed_value = 10.0
        with self.assertRaises(InvalidMinimumError):
            parameter.minimum_allowed_value = 11.0

        # Also check that when we set a value it falls within [min, max]
        parameter.value = 5

    def test_set_maximum_allowed_value(self):
        """Test setter for maximum allowed value."""
        parameter = FloatParameter()

        with self.assertRaises(TypeError):
            parameter.maximum_allowed_value = 'One million'

        parameter.maximum_allowed_value = 1000

        parameter.maximum_allowed_value = 11.0
        # Also check that it raises an error if it precedes min
        parameter.minimum_allowed_value = 10.0

        with self.assertRaises(InvalidMaximumError):
            parameter.maximum_allowed_value = 1.0

        # Also check that when we set a value it falls within [min, max]
        parameter.value = 10.5



# coding=utf-8
"""Select Parameter Widget"""

from PyQt4.QtCore import Qt
from PyQt4.QtGui import QComboBox

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget

__author__ = 'ismailsunni'
__project_name__ = 'parameters'
__filename__ = 'select_parameter_widget'
__date__ = '05/10/2016'
__copyright__ = 'imajimatika@gmail.com'


class SelectParameterWidget(GenericParameterWidget):
    """Widget class for List parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        :param parameter: A ListParameter object.
        :type parameter: ListParameter

        """
        super(SelectParameterWidget, self).__init__(parameter, parent)

        self.input = QComboBox()

        index = -1
        current_index = -1
        for opt in self._parameter.options_list:
            index += 1
            if opt == self._parameter.value:
                current_index = index
            self.input.addItem(opt)
            self.input.setItemData(index, opt, Qt.UserRole)

        self.input.setCurrentIndex(current_index)

        self.inner_input_layout.addWidget(self.input)

    def raise_invalid_type_exception(self):
        message = 'Expecting element type of %s' % (
            self._parameter.element_type.__name__)
        err = ValueError(message)
        return err

    def get_parameter(self):
        """Obtain list parameter object from the current widget state.

        :returns: A ListParameter from the current state of widget

        """
        current_index = self.input.currentIndex()
        selected_value = self.input.itemData(current_index, Qt.UserRole)
        if hasattr(selected_value, 'toPyObject'):
            selected_value = selected_value.toPyObject()

        try:
            self._parameter.value = selected_value
        except ValueError:
            err = self.raise_invalid_type_exception()
            raise err

        return self._parameter

    def set_choice(self, choice):
        """Set choice value by item's string.

        :param choice: The choice.
        :type choice: str

        :returns: True if success, else False.
        :rtype: bool
        """
        # Find index of choice
        choice_index = self._parameter.options_list.index(choice)
        if choice_index < 0:
            return False
        else:
            self.input.setCurrentIndex(choice_index)
            return True



# coding=utf-8

"""Collection Parameter abstract base class0."""
import abc

from generic_parameter import GenericParameter
from parameter_exceptions import (
    CollectionLengthError, InvalidMinimumError, InvalidMaximumError)


class CollectionParameter(GenericParameter):
    """A subclass of generic parameter that accepts collections only.

    This is a base class for List, Dict etc. parameters which share some
    functionality.

    .. versionadded:: 2.2
    """
    # Indicate that this is an abstract base class
    __metaclass__ = abc.ABCMeta

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(CollectionParameter, self).__init__(guid)
        self._minimum_item_count = None
        self._maximum_item_count = None
        self._element_type = None

    @property
    def minimum_item_count(self):
        """Property for minimum allowed item count."""
        return self._minimum_item_count

    @minimum_item_count.setter
    def minimum_item_count(self, minimum_count):
        """Define the minimum number of item in the parameter.

        :param minimum_count: Minimum number of items that may be
            selected. Defaults to 1.
        :type minimum_count: int
        """
        # If maximum is not set, we can set minimum regardless
        if self._maximum_item_count is None:
            self._minimum_item_count = minimum_count
            return
        # Otherwise it must be less than maximum
        if minimum_count <= self._maximum_item_count:
            self._minimum_item_count = minimum_count
            return

        self._minimum_item_count = minimum_count
        raise InvalidMinimumError(
            'Minimum item count must be no more than maximum')

    @property
    def maximum_item_count(self):
        """Property for maximum allowed item count."""
        return self._maximum_item_count

    @maximum_item_count.setter
    def maximum_item_count(self, maximum_count):
        """Define the maximum allowed number of items that can be selected.

        :param maximum_count: Maximum number of items that may be selected.
            Defaults to 1.
        :type maximum_count: int
        """
        # If maximum is not set, we can set minimum regardless
        if self._maximum_item_count is None:
            self._maximum_item_count = maximum_count
            return
        # Otherwise it must be less than maximum
        if maximum_count >= self._minimum_item_count:
            self._maximum_item_count = maximum_count
            return
        raise InvalidMaximumError('Maximum must be greater than minimum')

    def count(self):
        """Obtain the number of element in the list.

        :returns: The number of elements.
        :rtype: int
        """
        return len(self._value)

    @property
    def element_type(self):
        """Property for the allowed type for elements in the list.

        The element type determines if the list should be composed of strings,
        ints, floats etc.

        """
        return self._element_type

    @element_type.setter
    def element_type(self, element_type):
        """Define the allowed type for elements in the list.

        :param element_type: The element type determines if the list should be
            composed of strings, ints, floats etc.
        :type element_type: string, int, float
        """
        self._element_type = element_type

    def check_types(self, value):
        """
        Helper to check the type of the value and its elements.

        :param value: The list of values set for this parameter.
        :type value: str, bool, integer, float, list, dict

        :raises: TypeError
        """
        # Checking that the type of _value is the same as the expected _value
        if not isinstance(value, self.expected_type):
            message = (
                'The type of the value is [%s] but a [%s] is expected.' % (
                    str(type(value)), str(self.expected_type)))
            raise TypeError(message)

        if isinstance(value, dict):
            inspected_values = [value[key] for key in value.keys()]
        elif isinstance(value, list):
            inspected_values = value
        else:
            message = 'The value type is not a collection type'
            raise TypeError(message)

        self._check_sub_values(inspected_values)

    def _check_sub_values(self, values):
        for element in values:
            if isinstance(element, dict) or isinstance(element, list):
                self._check_sub_values(element)
            elif not isinstance(element, self.element_type):
                message = (
                    'The type of the element is [%s] but an [%s] is expected.'
                    % (str(type(element)), str(self.element_type)))
                raise TypeError(message)

    def check_length(self, value):
        """Check if the supplied container is within acceptable length limits.

        :raises: CollectionLengthError
        """

        if (self._maximum_item_count is None and
                self._minimum_item_count is None):
            return

        length = len(value)
        minimum = self.minimum_item_count
        maximum = self.maximum_item_count
        message = (
            '%i elements were found in the collection%s, there should be '
            'at least %i element(s) and no more than %i element(s).' %
            (
                length,
                '' if self.name is None else ' "' + self.name + '"',
                0 if minimum is None else minimum,
                999 if maximum is None else maximum
            ))
        # If maximum is not set, we can set minimum regardless
        if length > maximum:
            raise CollectionLengthError(message)
        # Also it must not be less than minimum
        if length < minimum:
            raise CollectionLengthError(message)

    @property
    def value(self):
        """Property for value of this parameter."""
        if self._value is None:
            self._value = []
        return self._value

    @value.setter
    def value(self, value):
        """Define the current value for the parameter.

        Need to check the type of each element.

        :param value: The collection of values set for this parameter.
        :type value: dict, list

        :raises: TypeError, CollectionLengthError
        """
        self.check_types(value)
        self.check_length(value)
        self._value = value

    def __len__(self):
        return self.value.__len__()

    def __getitem__(self, i):
        return self.value[i]

    def __setitem__(self, i, val):
        return self.value.__setitem__(i, val)



# coding=utf-8
"""Main file for showing off widget parameter."""

import sys
from functools import partial

from PyQt4.QtGui import (
    QApplication, QWidget, QGridLayout, QPushButton, QMessageBox)
from boolean_parameter import BooleanParameter
from dict_parameter import DictParameter
from float_parameter import FloatParameter
from group_parameter import GroupParameter
from input_list_parameter import InputListParameter
from integer_parameter import IntegerParameter
from metadata import unit_feet_depth, unit_metres_depth
from qt_widgets.input_list_parameter_widget import InputListParameterWidget
from qt_widgets.parameter_container import ParameterContainer
from qt_widgets.test.custom_parameter.point_parameter import PointParameter
from qt_widgets.test.custom_parameter.point_parameter_widget import (
    PointParameterWidget)
from select_parameter import SelectParameter
from string_parameter import StringParameter
from unit import Unit

from parameters.list_parameter import ListParameter

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'main'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'


def main():
    """Main function"""
    app = QApplication([])

    def validate_min_max(parent_container):
        """
        :param parent_container: The container that use this validator.
        :type parent_container: ParameterContainer
        :return:
        """
        min_value_parameter = parent_container.get_parameter_by_guid(
            min_integer_parameter.guid)
        max_value_parameter = parent_container.get_parameter_by_guid(
            max_integer_parameter.guid)

        min_value = min_value_parameter.value
        max_value = max_value_parameter.value

        print 'min', min_value
        print 'max', max_value

        if min_value > max_value:
            print 'Not valid'
            return {
                'valid': False,
                'message': (
                    'Your minimum value (%d) should be less than your maximum '
                    'value (%d)' % (min_value, max_value))
            }
        print 'Valid'
        return {'valid': True, 'message': ''}

    unit_feet = Unit('130790')
    unit_feet.load_dictionary(unit_feet_depth)

    unit_metres = Unit('900713')
    unit_metres.load_dictionary(unit_metres_depth)

    string_parameter = StringParameter('28082014')
    string_parameter.name = 'Province Name'
    string_parameter.description = 'Name of province.'
    string_parameter.help_text = (
        'A <b>test help</b> that is very long so that you need to '
        'read it for one minute and you will be tired after read this '
        'description. You are the best user so far. Even better if you read '
        'this description loudly so that all of your friends will be able '
        'to hear you')
    string_parameter.is_required = True
    string_parameter.value = 'Daerah Istimewa Yogyakarta'

    boolean_parameter = BooleanParameter('1231231')
    boolean_parameter.name = 'Post processor'
    boolean_parameter.description = 'This is post processor parameter.'
    boolean_parameter.help_text = (
        'A <b>test help text</b> that is very long so that you need to '
        'read it for one minute and you will be tired after read this '
        'description. You are the best user so far. Even better if you read '
        'this description loudly so that all of your friends will be able '
        'to hear you')
    boolean_parameter.is_required = True
    boolean_parameter.value = True

    float_parameter = FloatParameter()
    float_parameter.name = 'Flood Depth'
    float_parameter.is_required = True
    float_parameter.precision = 3
    float_parameter.minimum_allowed_value = 1.0
    float_parameter.maximum_allowed_value = 2.0
    float_parameter.description = 'The depth of flood.'
    float_parameter.help_text = (
        'A <b>test _description</b> that is very long so that you need to '
        'read it for one minute and you will be tired after read this '
        'description. You are the best user so far. Even better if you read '
        'this description loudly so that all of your friends will be able '
        'to hear you')
    float_parameter.unit = unit_feet
    float_parameter.allowed_units = [unit_metres, unit_feet]
    float_parameter.value = 1.12

    integer_parameter = IntegerParameter()
    integer_parameter.name = 'Paper'
    integer_parameter.is_required = True
    integer_parameter.minimum_allowed_value = 1
    integer_parameter.maximum_allowed_value = 5
    integer_parameter.description = 'Number of paper'
    integer_parameter.help_text = (
        'A <b>test _description</b> that is very long so that you need to '
        'read it for one minute and you will be tired after read this '
        'description. You are the best user so far. Even better if you read '
        'this description loudly so that all of your friends will be able '
        'to hear you')
    integer_parameter.unit = unit_feet
    integer_parameter.allowed_units = [unit_feet]
    integer_parameter.value = 3

    point_parameter = PointParameter()
    point_parameter.name = 'Point Parameter'
    point_parameter.is_required = True
    point_parameter.description = 'Short help.'
    point_parameter.help_text = 'Long description for parameter.'
    point_parameter.value = (0, 1)

    min_integer_parameter = IntegerParameter()
    min_integer_parameter.name = 'Minimal Stick Length'
    min_integer_parameter.is_required = True
    min_integer_parameter.minimum_allowed_value = 1
    min_integer_parameter.maximum_allowed_value = 50
    min_integer_parameter.description = 'Minimum length of a stick'
    min_integer_parameter.help_text = (
        'Minimum length of a stick that are allowed')
    min_integer_parameter.unit = unit_metres
    min_integer_parameter.allowed_units = [unit_metres]
    min_integer_parameter.value = 3

    max_integer_parameter = IntegerParameter()
    max_integer_parameter.name = 'Maximum Stick Length'
    max_integer_parameter.is_required = True
    max_integer_parameter.minimum_allowed_value = 1
    max_integer_parameter.maximum_allowed_value = 50
    max_integer_parameter.description = 'Maximum length of a stick'
    max_integer_parameter.help_text = (
        'Maximum length of a stick that are allowed')
    max_integer_parameter.unit = unit_metres
    max_integer_parameter.allowed_units = [unit_metres]
    max_integer_parameter.value = 4

    list_parameter = ListParameter()
    list_parameter.name = 'Affected Field'
    list_parameter.is_required = True
    list_parameter.maximum_item_count = 3
    list_parameter.minimum_item_count = 1
    list_parameter.description = 'Column used for affected field'
    list_parameter.help_text = 'Column used for affected field in the vector'
    list_parameter.element_type = str
    list_parameter.options_list = ['FLOODPRONE', 'affected', 'floodprone',
                                   'yes/no', '\xddounicode test']
    list_parameter.value = ['FLOODPRONE', 'affected', 'floodprone']

    select_parameter = SelectParameter()
    select_parameter.name = 'Select Affected Field'
    select_parameter.is_required = True
    select_parameter.description = 'Column used for affected field'
    select_parameter.help_text = (
        'Column used for affected field in the vector')
    select_parameter.element_type = str
    select_parameter.options_list = [
        'FLOODPRONE', 'affected', 'floodprone', 'yes/no', '\xddounicode test']
    select_parameter.value = 'affected'

    input_list_parameter = InputListParameter()
    input_list_parameter.name = 'Thresholds'
    input_list_parameter.is_required = True
    input_list_parameter.maximum_item_count = 3
    input_list_parameter.minimum_item_count = 1
    input_list_parameter.description = 'Specified List of thresholds'
    input_list_parameter.help_text = 'Some help text'
    input_list_parameter.element_type = int
    input_list_parameter.ordering = InputListParameter.DescendingOrder
    input_list_parameter.value = [1]

    dict_parameter = DictParameter()
    dict_parameter.name = 'Dict Parameter'
    dict_parameter.is_required = True
    dict_parameter.maximum_item_count = 5
    dict_parameter.minimum_item_count = 1
    dict_parameter.description = 'Dict Parameter example'
    dict_parameter.help_text = 'Dict Parameter help text.'
    dict_parameter.element_type = str
    dict_parameter.value = {
        'foo': 'True',
        'bar': '10',
        'woo': 'False',
        'sub_dict_sample': {
            'key1': 'val1',
            'key2': 'val2'
        }
    }

    group_parameter = GroupParameter()
    group_parameter.name = 'Age ratios'
    group_parameter.is_required = True
    group_parameter.value = [
        string_parameter,
        integer_parameter,
        boolean_parameter
    ]

    def _custom_validator(value):
        valid = True
        if string_parameter.value == 'foo' and integer_parameter.value == \
                3 and boolean_parameter.value is True:
            valid = False
        if not valid:
            raise Exception('Parameter not valid')

    group_parameter.custom_validator = _custom_validator

    parameters = [
        string_parameter,
        integer_parameter,
        boolean_parameter,
        float_parameter,
        float_parameter,
        boolean_parameter,
        integer_parameter,
        point_parameter,
        list_parameter,
        input_list_parameter,
        dict_parameter,
        group_parameter,
        select_parameter
    ]

    extra_parameters = [
        (PointParameter, PointParameterWidget)
    ]
    min_max_parameters = [min_integer_parameter, max_integer_parameter]

    description_text = (
        'These parameters are merely created for showing example only')
    # description_text = ''
    parameter_container = ParameterContainer(
        parameters,
        extra_parameters=extra_parameters,
        description_text=description_text)
    parameter_container.setup_ui()

    # create error handler
    parameter_widget = parameter_container.get_parameter_widgets()
    try:
        input_list_widget = [
            w.widget() for w in parameter_widget if
            isinstance(w.widget(), InputListParameterWidget)][0]

        def add_row_handler(exception):
            box = QMessageBox()
            box.critical(input_list_widget, 'Add Row Error', exception.message)

        input_list_widget.add_row_error_handler = add_row_handler
    except IndexError:
        pass

    parameter_container2 = ParameterContainer(
        extra_parameters=extra_parameters,
        description_text='Empty Parameter Container Description')
    parameter_container2.setup_ui()

    parameter_container3 = ParameterContainer(
        parameters=min_max_parameters,
        extra_parameters=extra_parameters,
        description_text='Minimum Maximum Parameter')
    parameter_container3.add_validator(validate_min_max)
    parameter_container3.setup_ui()

    def show_error_message(parent, exception):
        """Generate error message to handle parameter errors

        :param parent: The widget as a parent of message box
        :type parent: QWidget
        :param exception: python Exception or Error
        :type exception: Exception
        """
        box = QMessageBox()
        box.critical(parent, 'Error occured', exception.message)

    def show_parameter(the_parameter_container):
        """Print the value of parameter.

        :param the_parameter_container: A parameter container
        :type the_parameter_container: ParameterContainer
        """
        def show_parameter_value(a_parameter):
            if isinstance(a_parameter, GroupParameter):
                for param in a_parameter.value:
                    show_parameter_value(param)
            else:
                print a_parameter.guid, a_parameter.name, a_parameter.value

        try:
            the_parameters = the_parameter_container.get_parameters()
            if the_parameters:
                for parameter in the_parameters:
                    show_parameter_value(parameter)
        except Exception as inst:
            show_error_message(parameter_container, inst)

    button = QPushButton('Show parameters')
    # noinspection PyUnresolvedReferences
    button.clicked.connect(
        partial(show_parameter, the_parameter_container=parameter_container))

    validate_button = QPushButton('Validate parameters')
    # noinspection PyUnresolvedReferences
    validate_button.clicked.connect(
        partial(show_parameter, the_parameter_container=parameter_container3))

    widget = QWidget()
    layout = QGridLayout()
    layout.addWidget(parameter_container)
    layout.addWidget(button)
    layout.addWidget(parameter_container2)
    layout.addWidget(parameter_container3)
    layout.addWidget(validate_button)

    widget.setLayout(layout)
    widget.setGeometry(0, 0, 500, 500)

    widget.show()

    sys.exit(app.exec_())

if __name__ == '__main__':
    main()



# coding=utf-8
"""List Parameter."""

from collection_parameter import CollectionParameter
from parameter_exceptions import ValueNotAllowedException


class ListParameter(CollectionParameter):
    """A subclass of parameter that allows the user to select from a list.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(ListParameter, self).__init__(guid)
        self.expected_type = list
        self._options_list = None
        self._is_editable = False

    @property
    def is_editable(self):
        """Flag to determine that the list items is editable"""
        return self._is_editable

    @is_editable.setter
    def is_editable(self, value):
        self._is_editable = value

    @property
    def options_list(self):
        """Stores the list of options the value can take"""
        return self._options_list

    @options_list.setter
    def options_list(self, value):
        # the options type must be the same as the value type
        self.check_types(value)
        self._options_list = value

    @property
    def value(self):
        if self._value is None:
            self._value = []
        return self._value

    @value.setter
    def value(self, value):
        """Define the current value for the parameter.

        Need to check the type of each element.

        :param value: The collection of values set for this parameter.
        :type value: dict, list

        :raises: TypeError, CollectionLengthError
        """
        self.check_types(value)
        self.check_length(value)
        self.check_value_in_options(value)
        self._value = value

    def check_value_in_options(self, value):
        """Check items of value assigned is a subset of options_list

        :param value: The collection of values checked
        :type value: dict, list

        :raise: ValueNotAllowedException
        """
        message = 'The value %s doesn\'t exists on the allowed options list.'
        for val in value:
            if val not in self.options_list:
                message = message % val
                raise ValueNotAllowedException(message)



# coding=utf-8
"""Test class for select_parameter_widget."""

import unittest

from PyQt4.QtGui import QApplication

from parameters.qt_widgets.select_parameter_widget import SelectParameterWidget
from parameters.select_parameter import SelectParameter


class TestSelectParameterWidget(unittest.TestCase):
    """Test for SelectParameterWidget"""
    application = QApplication([])

    def test_init(self):
        select_parameter = SelectParameter()
        select_parameter.name = 'Select Affected Field'
        select_parameter.is_required = True
        select_parameter.help_text = 'Column used for affected field'
        select_parameter.description = (
            'Column used for affected field in the vector')
        select_parameter.element_type = str
        select_parameter.options_list = [
            'FLOODPRONE', 'affected', 'floodprone', 'yes/no',
            '\xddounicode test']
        select_parameter.value = 'affected'

        widget = SelectParameterWidget(select_parameter)

        expected_value = select_parameter.value
        real_value = widget.get_parameter().value
        self.assertEqual(expected_value, real_value)

        widget.input.setCurrentIndex(0)
        real_value = widget.get_parameter().value
        self.assertEqual(real_value, select_parameter.options_list[0])

    def test_set_choice(self):
        """Test for set_choice method."""
        select_parameter = SelectParameter()
        select_parameter.name = 'Select Affected Field'
        select_parameter.is_required = True
        select_parameter.help_text = 'Column used for affected field'
        select_parameter.description = (
            'Column used for affected field in the vector')
        select_parameter.element_type = str
        select_parameter.options_list = [
            'FLOODPRONE', 'affected', 'floodprone', 'yes/no',
            '\xddounicode test']
        select_parameter.value = 'affected'

        widget = SelectParameterWidget(select_parameter)

        widget.set_choice('floodprone')
        real_value = widget.get_parameter().value
        self.assertEqual('floodprone', real_value)



# coding=utf-8
"""A registry implementation for parameters."""


class Registry(object):
    """A simple registry for keeping track of all parameters.

    We will use a singleton pattern to ensure that there is only
    one canonical registry. The registry can be used by parameters
    to register themselves and their GUID's.
    """

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Registry, cls).__new__(
                cls, *args, **kwargs)
        return cls._instance



# coding=utf-8
"""Generic Parameter Widget for this file."""

from PyQt4.QtGui import (
    QWidget,
    QHBoxLayout,
    QVBoxLayout,
    QLabel,
    QToolButton,
    QGridLayout,
    QSizePolicy)

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'generic_parameter_widget'
__date__ = '8/21/14'
__copyright__ = 'ismail@kartoza.com'


class GenericParameterWidget(QWidget, object):
    """Widget class for generic parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A Generic object.
        :type parameter: GenericParameter

        """
        QWidget.__init__(self, parent)
        self._parameter = parameter

        # Create elements
        # Label (name)
        self.label = QLabel(self._parameter.name)

        # Label (description text)
        # Hacky fix for #1830 - checking the base type
        if isinstance(self._parameter.description, basestring):
            self.description_text_label = QLabel(self._parameter.description)
        else:
            self.description_text_label = QLabel()

        self.description_text_label.setWordWrap(True)

        # Label (help)
        self.help_label = QLabel(self._parameter.help_text)
        self.help_label.setWordWrap(True)
        self.help_label.hide()

        # Flag for show-status of help
        self._hide_help = True

        # Tool button for showing and hide detail help
        self.switch_button = QToolButton()
        self.switch_button.setArrowType(4)  # 2=down arrow, 4=right arrow
        # noinspection PyUnresolvedReferences
        self.switch_button.clicked.connect(self.show_hide_help)
        self.switch_button.setToolTip('Click for detail help')
        self.switch_button_stylesheet = 'border: none;'
        self.switch_button.setStyleSheet(self.switch_button_stylesheet)
        # Layouts
        self.main_layout = QVBoxLayout()
        self.input_layout = QHBoxLayout()
        self.help_layout = QGridLayout()
        # _inner_input_layout must be filled with widget in the child class
        self.inner_input_layout = QHBoxLayout()
        self.inner_help_layout = QVBoxLayout()

        # spacing
        self.main_layout.setSpacing(0)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.input_layout.setSpacing(0)
        self.input_layout.setContentsMargins(0, 0, 0, 0)
        self.help_layout.setSpacing(0)
        self.help_layout.setContentsMargins(0, 0, 0, 0)
        self.inner_input_layout.setSpacing(7)
        self.inner_input_layout.setContentsMargins(0, 0, 0, 0)
        self.inner_help_layout.setSpacing(0)
        self.inner_help_layout.setContentsMargins(0, 0, 0, 0)

        # Put elements into layouts
        self.input_layout.addWidget(self.label)
        self.input_layout.addLayout(self.inner_input_layout)

        if self._parameter.description:
            self.help_layout.addWidget(self.description_text_label, 0, 1)
        if self._parameter.help_text:
            self.help_layout.addWidget(self.switch_button, 0, 0)
            self.help_layout.addWidget(self.help_label, 1, 1)

        self.main_layout.addLayout(self.input_layout)
        self.main_layout.addLayout(self.help_layout)

        self.setLayout(self.main_layout)

        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.MinimumExpanding)

    def get_parameter(self):
        """Interface for returning parameter object.

        This must be implemented in child class.

        :raises: NotImplementedError

        """
        raise NotImplementedError('Must be implemented in child class')

    def show_hide_help(self):
        """Show and hide long help."""
        if self._hide_help:
            self._hide_help = False
            self.help_label.show()
            self.switch_button.setArrowType(2)
            self.switch_button.setToolTip('Click for hide detail help')
        else:
            self._hide_help = True
            self.help_label.hide()
            self.switch_button.setArrowType(4)
            self.switch_button.setToolTip('Click for detail help')



# coding=utf-8
"""QT4 Parameter Factory."""

from parameters.qt_widgets.boolean_parameter_widget import BooleanParameterWidget
from parameters.qt_widgets.dict_parameter_widget import DictParameterWidget
from parameters.qt_widgets.float_parameter_widget import FloatParameterWidget
from parameters.qt_widgets.group_parameter_widget import GroupParameterWidget
from parameters.qt_widgets.input_list_parameter_widget import InputListParameterWidget
from parameters.qt_widgets.integer_parameter_widget import IntegerParameterWidget
from parameters.qt_widgets.list_parameter_widget import ListParameterWidget
from parameters.qt_widgets.select_parameter_widget import SelectParameterWidget
from parameters.qt_widgets.text_parameter_widget import TextParameterWidget

from parameters.qt_widgets.string_parameter_widget import StringParameterWidget

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'qt4_parameter_factory'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'


class Qt4ParameterFactory(object):
    """A factory class that will generate a widget given a parameter."""

    def __init__(self):
        """Constructor."""
        self.dict_widget = {
            'BooleanParameter': BooleanParameterWidget,
            'FloatParameter': FloatParameterWidget,
            'IntegerParameter': IntegerParameterWidget,
            'StringParameter': StringParameterWidget,
            'ListParameter': ListParameterWidget,
            'InputListParameter': InputListParameterWidget,
            'DictParameter': DictParameterWidget,
            'TextParameter': TextParameterWidget,
            'GroupParameter': GroupParameterWidget,
            'SelectParameter': SelectParameterWidget
        }

    def register_widget(self, parameter, parameter_widget):
        """Register new custom widget.

        :param parameter:
        :type parameter: GenericParameter

        :param parameter_widget:
        :type parameter_widget: GenericParameterWidget
        """
        self.dict_widget[parameter.__name__] = parameter_widget

    def remove_widget(self, parameter):
        """Register new custom widget.

        :param parameter:
        :type parameter: GenericParameter
        """
        if parameter.__name__ in self.dict_widget.keys():
            self.dict_widget.pop(parameter.__name__)

    def get_widget(self, parameter):
        """Create parameter widget from current
        :param parameter: Parameter object.
        :type parameter: BooleanParameter, FloatParameter, IntegerParameter,
            StringParameter

        :returns: Widget of given parameter.
        :rtype: BooleanParameterWidget, FloatParameterWidget,
            IntegerParameterWidget, StringParameterWidget
        """
        class_name = parameter.__class__.__name__

        if class_name in self.dict_widget.keys():
            return self.dict_widget[class_name](parameter)
        else:
            raise TypeError(class_name)

    @staticmethod
    def get_parameter(widget):
        """Obtain parameter object from parameter widget current state.

        :param widget: An object of a ParameterWidget.
        :type widget: GenericParameterWidget

        :returns: Parameter object
        :rtype: GenericParameter

        """
        return widget.get_parameter()



# coding=utf-8
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'boolean_parameter_widget'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'

from PyQt4.QtGui import QCheckBox

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget


class BooleanParameterWidget(GenericParameterWidget):
    """Widget class for boolean parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A BooleanParameter object.
        :type parameter: BooleanParameter

        """
        super(BooleanParameterWidget, self).__init__(parameter, parent)

        # Get the parameter label and use its value as the checkbox text
        label_item = self.input_layout.itemAt(0)
        label_widget = label_item.widget()
        text = label_widget.text()

        self._check_box_input = QCheckBox(text)
        # Tooltips
        self.setToolTip('Tick here to enable ' + self._parameter.name)
        self._check_box_input.setChecked(self._parameter.value)

        self.inner_input_layout.insertWidget(0, self._check_box_input)
        self.input_layout.removeItem(label_item)

    def get_parameter(self):
        """Obtain boolean parameter object from the current widget state.

        :returns: A BooleanParameter from the current state of widget

        """
        self._parameter.value = self._check_box_input.isChecked()
        return self._parameter



# coding=utf-8
"""Test class for integer_parameter_widget."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'test_integer_parameter_widget'
__date__ = '8/21/14'
__copyright__ = 'ismail@kartoza.com'

import unittest

from PyQt4.QtGui import QApplication

from parameters.metadata import unit_metres_depth, unit_feet_depth
from parameters.qt_widgets.integer_parameter_widget import IntegerParameterWidget
from parameters.unit import Unit

from parameters.integer_parameter import IntegerParameter


class TestFloatParameterWidget(unittest.TestCase):
    application = QApplication([])

    def test_init(self):

        unit_feet = Unit('130790')
        unit_feet.load_dictionary(unit_feet_depth)

        unit_metres = Unit('900713')
        unit_metres.load_dictionary(unit_metres_depth)

        integer_parameter = IntegerParameter()
        integer_parameter.name = 'Paper'
        integer_parameter.is_required = True
        integer_parameter.minimum_allowed_value = 1
        integer_parameter.maximum_allowed_value = 5
        integer_parameter.help_text = 'Number of paper'
        integer_parameter.description = (
            'A <b>test _description</b> that is very long so that you need '
            'to read it for one minute and you will be tired after read this '
            'description. You are the best user so far. Even better if you '
            'read this description loudly so that all of your friends will be '
            'able to hear you')
        integer_parameter.unit = unit_feet
        integer_parameter.allowed_units = [unit_feet]
        integer_parameter.value = 3

        widget = IntegerParameterWidget(integer_parameter)

        expected_value = integer_parameter.name
        real_value = widget.label.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        expected_value = integer_parameter.value
        real_value = widget.get_parameter().value
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        widget._input.setValue(1.5)

        expected_value = 1
        real_value = widget.get_parameter().value
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        widget._input.setValue(1.55555)

        expected_value = 1
        real_value = widget.get_parameter().value
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        widget._input.setValue(7)

        expected_value = 5
        real_value = widget.get_parameter().value
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        expected_value = 'QLabel'
        real_value = widget._unit_widget.__class__.__name__
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        expected_value = 'feet'
        real_value = widget._unit_widget.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

if __name__ == '__main__':
    unittest.main()



# coding=utf-8
"""Test class for string_parameter_widget."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'test_string_parameter_widget.py'
__date__ = '8/28/14'
__copyright__ = 'ismail@kartoza.com'

import unittest

from PyQt4.QtGui import QApplication
from parameters.string_parameter import StringParameter

from parameters.qt_widgets.string_parameter_widget import StringParameterWidget


class TestBooleanParameterWidget(unittest.TestCase):
    application = QApplication([])

    def test_init(self):
        string_parameter = StringParameter('28082014')
        string_parameter.name = 'Province Name'
        string_parameter.help_text = 'Name of province.'
        string_parameter.description = (
            'A <b>test _description</b> that is very long so that you need to '
            'read it for one minute and you will be tired after read this '
            'description. You are the best user so far. Even better if you '
            'read this description loudly so that all of your friends will '
            'be able to hear you')
        string_parameter.is_required = True
        string_parameter.value = 'Daerah Istimewa Yogyakarta'

        widget = StringParameterWidget(string_parameter)

        expected_value = string_parameter.name
        real_value = widget.label.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        expected_value = string_parameter.value
        real_value = widget._line_edit_input.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        # change value
        widget._line_edit_input.setText('Nusa Tenggara Barat')

        expected_value = 'Nusa Tenggara Barat'
        real_value = widget._line_edit_input.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

if __name__ == '__main__':
    unittest.main()



# coding=utf-8

__author__ = 'lucernae'
__project_name__ = 'parameters'
__filename__ = 'list_parameter_widget'
__date__ = '02/04/15'
__copyright__ = 'lana.pcfre@gmail.com'

from PyQt4.QtCore import Qt
from PyQt4.QtGui import (
    QListWidget,
    QAbstractItemView,
    QListWidgetItem,
    QVBoxLayout)

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget


class ListParameterWidget(GenericParameterWidget):
    """Widget class for List parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A ListParameter object.
        :type parameter: ListParameter

        """
        super(ListParameterWidget, self).__init__(parameter, parent)

        self._input = QListWidget()

        self._input.setSelectionMode(QAbstractItemView.MultiSelection)

        if self._parameter.maximum_item_count != \
                self._parameter.minimum_item_count:
            tool_tip = 'Select between %d and %d items' % (
                self._parameter.minimum_item_count,
                self._parameter.maximum_item_count)
        else:
            tool_tip = 'Select exactly %d items' % (
                       self._parameter.maximum_item_count)

        self._input.setToolTip(tool_tip)

        for opt in self._parameter.options_list:
            item = QListWidgetItem()
            item.setFlags(item.flags() | Qt.ItemIsEditable)
            item.setText(str(opt))
            self._input.addItem(item)
            if opt in self._parameter.value:
                item.setSelected(True)

        self.inner_input_layout.addWidget(self._input)

        # override self._input_layout arrangement to make the label at the top
        # reset the layout
        self.input_layout.setParent(None)
        self.help_layout.setParent(None)

        self.label.setParent(None)
        self.inner_input_layout.setParent(None)

        self.input_layout = QVBoxLayout()
        self.input_layout.setSpacing(0)

        # put element into layout
        self.input_layout.addWidget(self.label)
        self.input_layout.addLayout(self.inner_input_layout)

        self.main_layout.addLayout(self.input_layout)
        self.main_layout.addLayout(self.help_layout)

    def raise_invalid_type_exception(self):
        message = 'Expecting element type of %s' % (
            self._parameter.element_type.__name__)
        err = ValueError(message)
        return err

    def get_parameter(self):
        """Obtain list parameter object from the current widget state.

        :returns: A ListParameter from the current state of widget

        """
        selected_value = []
        for opt in self._input.selectedItems():
            index = self._input.indexFromItem(opt)
            selected_value.append(self._parameter.options_list[index.row()])

        try:
            self._parameter.value = selected_value
        except ValueError:
            err = self.raise_invalid_type_exception()
            raise err

        return self._parameter



# coding=utf-8
"""List Parameter."""

from collection_parameter import CollectionParameter


class InputListParameter(CollectionParameter):
    """A subclass of parameter that allows the user to select from a list.

    .. versionadded:: 2.2
    """

    NotOrdered = 'NotOrdered'
    AscendingOrder = 'Ascending'
    DescendingOrder = 'Descending'

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(InputListParameter, self).__init__(guid)
        self.expected_type = list
        self._ordering = InputListParameter.NotOrdered

    @property
    def ordering(self):
        return self._ordering

    @ordering.setter
    def ordering(self, value):
        self._ordering = value

    @property
    def value(self):
        if self._value is None:
            self._value = []
        return self._value

    @value.setter
    def value(self, value):
        self.check_types(value)
        self.check_length(value)
        self._value = value



# coding=utf-8
"""List Parameter."""

from collection_parameter import CollectionParameter
from generic_parameter import GenericParameter


class GroupParameter(CollectionParameter):
    """A subclass of parameter that allows the user to select from a list.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(GroupParameter, self).__init__(guid)
        self.expected_type = list
        self.element_type = GenericParameter
        self.is_required = False
        self._enable_parameter = True
        self._custom_validator = None
        self._must_scroll = True

    @property
    def enable_parameter(self):
        return self._enable_parameter

    @enable_parameter.setter
    def enable_parameter(self, value):
        self._enable_parameter = value

    @property
    def must_scroll(self):
        return self._must_scroll

    @must_scroll.setter
    def must_scroll(self, value):
        self._must_scroll = value

    @property
    def custom_validator(self):
        return self._custom_validator

    @custom_validator.setter
    def custom_validator(self, value):
        """
        :param value: function object with one argument for the parameter
        :type value: (list[Parameter]) -> None
        """
        self._custom_validator = value

    def validate(self):
        if self.custom_validator:
            self.custom_validator(self.value)



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'unit'
__date__ = '8/25/14'
__copyright__ = 'ismail@kartoza.com'

import uuid


class Unit(object):
    """Class for representing unit."""
    def __init__(self, guid=None):
        """Constructor

        :param guid: Optional unique identifier for this unit. If none
            is specified one will be generated using python hash. This guid
            will be used when storing unit in the registry.
        :type guid: str, None

        """
        self.guid = None
        if guid is None:
            self._guid = uuid.uuid4()
        else:
            self._guid = guid
        self._name = None
        self._help_text = None
        self._description = None
        self._plural = None
        self._abbreviation = None

    @property
    def guid(self):
        """Unique identifier property for a unit.

        :returns: A globally unique identifier for the unit.
        :rtype: str
        """
        return self._guid

    @guid.setter
    def guid(self, guid):
        """Assign a unique identifier to the unit instance.

        :param guid: A globally unique identified (never translated).
        :type guid: str

        Perhaps it should set the guid itself?
        """
        self._guid = guid

    @property
    def name(self):
        """Name property for the unit.

        :returns: The name for this unit.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Set the name for the unit.

        :param name: The name for the units. This will be used in the UI,
            and can be translated)
        :type name: str
        """
        self._name = name

    @property
    def plural(self):
        """The plural name property of the unit.

        :returns: The plural name for this unit.
        :rtype: str
        """
        return self._plural

    @plural.setter
    def plural(self, plural):
        """Set the plural name for the unit.

        :param plural: The plural name for this unit.
        :type plural: str
        """
        self._plural = plural

    @property
    def abbreviation(self):
        """The abbreviated name property of the unit.

        :returns: The abbreviated name for this unit.
        :rtype: str
        """
        return self._abbreviation

    @abbreviation.setter
    def abbreviation(self, abbreviation):
        """Set the abbreviated name for the unit.

        :param abbreviation: The abbreviated name for this unit.
        :type abbreviation: str
        """
        self._abbreviation = abbreviation

    @property
    def help_text(self):
        """Property containing help text for the parameter.

        It will be used as tooltip in the UI..

        :returns: A short help sentence for the parameter.
        :rtype: str
        """
        return self._help_text

    @help_text.setter
    def help_text(self, help_text):
        """Define the help help_text for this parameter.

        :param help_text: A short (i.e. one line) explanation of the parameter.
        :type help_text: str
        """
        self._help_text = help_text

    @property
    def description(self):
        """Property for the description of the unit.

        :returns: A detailed description for the unit.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Define the description for this unit.

        :param description: A detailed description of the unit
        :type description: str
        """
        self._description = description

    def load_dictionary(self, unit_dictionary):
        """Load from unit dictionary to initialize unit object

        :param unit_dictionary: Unit dictionary, the same in the metadata.py
        :type unit_dictionary: dict

        """
        name = unit_dictionary.get('name', '')
        plural = unit_dictionary.get('plural', '')
        abbreviation = unit_dictionary.get('abbreviation', '')
        help_text = unit_dictionary.get('help_text', '')
        description = unit_dictionary.get('description', '')

        if name:
            self.name = name
        if plural:
            self.plural = plural
        if abbreviation:
            self.abbreviation = abbreviation
        if help_text:
            self.help_text = help_text
        if description:
            self.description = description

    def serialize(self):
        """Serialize the unit.

        :returns: The unit content in a dict format
        :rtype: dict
        """
        return {
            'name': self.name,
            'plural': self.plural,
            'abbreviation': self.abbreviation,
            'help_text': self.help_text,
            'description': self.description,
            }

    def __str__(self):
        return self.name



# coding=utf-8
"""Exceptions related to parameters implementation."""


class InvalidMinimumError(Exception):
    """Error raised when minimum is not valid e.g. exceeds maximum."""
    pass


class InvalidMaximumError(Exception):
    """Error raised when maximum is not valid e.g. less than minimum."""
    pass


class ValueOutOfBounds(Exception):
    """Error raised when a value is outside allowed range of min, max."""
    pass


class CollectionLengthError(Exception):
    """Error raised when a collection is too long or too short."""
    pass


class RequiredException(Exception):
    """Exception raised when a parameter is required but not assigned.
    """
    pass


class InvalidValidationException(Exception):
    """Exception raised when there is validation breaker in ParameterContainer.
    """
    pass


class ValueNotAllowedException(Exception):
    """Exception raised when the value assigned in the ListParameter contains
    items that didn't exists in options list
    """
    pass



# coding=utf-8
"""Dictionary Parameter."""
from collection_parameter import CollectionParameter


class DictParameter(CollectionParameter):
    """A subclass of parameter that allows the user to select from a dict.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(DictParameter, self).__init__(guid)
        self.expected_type = dict

    def __setitem__(self, key, value):
        """Helper method to make DictParameter behave like dict"""
        self._value[key] = value

    def __getitem__(self, key):
        """Helper method to make DictParameter behave like dict"""
        return self._value[key]

    def keys(self):
        """Helper method to make DictParameter behave like dict"""
        return self._value.keys()

    @property
    def value(self):
        """Property for value of this parameter."""
        return self._value

    @value.setter
    def value(self, value):
        """Define the current value for the parameter.

        Need to check the type of each element isdict.

        :param value: The collection of values set for this parameter.
        :type value: dict

        :raises: TypeError
        """
        self.check_types(value)
        self.check_length(value)
        self._value = value



# coding=utf-8
"""Group Parameter Widget."""

from PyQt4.QtGui import QVBoxLayout, QCheckBox

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget

__author__ = 'lucernae'
__project_name__ = 'parameters'
__filename__ = 'list_parameter_widget'
__date__ = '02/04/15'
__copyright__ = 'lana.pcfre@gmail.com'


class GroupParameterWidget(GenericParameterWidget):
    """Widget class for List parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A GroupParameter object.
        :type parameter: GroupParameter

        """
        super(GroupParameterWidget, self).__init__(parameter, parent)

        # Get the parameter label and use its value as the checkbox text
        label_item = self.input_layout.itemAt(0)
        label_widget = label_item.widget()
        text = label_widget.text()
        self._enable_check_box = QCheckBox(text)
        # Tooltips
        self.setToolTip('Tick here to enable ' + self._parameter.name)

        # add all widget in the group
        self._group_layout = QVBoxLayout()
        self._group_layout.setSpacing(0)

        if not self._parameter.is_required:
            self.input_layout.insertWidget(0, self._enable_check_box)
            # now we don't need the parameter label anymore so chuck it
            self.input_layout.removeItem(label_item)
            # Make the sub group appear indented
            self._group_layout.setContentsMargins(20, 0, 0, 0)
        else:
            self._parameter.enable_parameter = True

        self.main_layout.addLayout(self._group_layout)

        # Why are we doing imports here? TS
        from parameters.qt_widgets.parameter_container import ParameterContainer

        self.param_container = ParameterContainer(
            parameters=self._parameter.value)
        self.param_container.setup_ui(must_scroll=parameter.must_scroll)

        # add handlers
        # noinspection PyUnresolvedReferences
        self._enable_check_box.stateChanged.connect(
            self.on_enable_checkbox_changed)
        self._enable_check_box.setChecked(self._parameter.enable_parameter)
        self.on_enable_checkbox_changed(self._parameter.enable_parameter)

        self._group_layout.addWidget(self.param_container)

    def on_enable_checkbox_changed(self, state):
        if state:
            self.param_container.show()
        else:
            self.param_container.hide()
        self._parameter.enable_parameter = state

    def get_parameter(self):
        """Obtain list parameter object from the current widget state.

        :returns: A ListParameter from the current state of widget

        """
        if self._parameter.enable_parameter:
            parameters = self.param_container.get_parameters()
            self._parameter.value = parameters
            self._parameter.validate()

        return self._parameter



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'string_parameter_widget'
__date__ = '8/28/14'
__copyright__ = 'ismail@kartoza.com'

from PyQt4.QtGui import QLineEdit, QSizePolicy

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget


class StringParameterWidget(GenericParameterWidget):
    """Widget class for string parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A StringParameter object.
        :type parameter: StringParameter

        """
        super(StringParameterWidget, self).__init__(parameter, parent)

        self._line_edit_input = QLineEdit()
        self._line_edit_input.setSizePolicy(
            QSizePolicy.Minimum, QSizePolicy.Fixed)
        # Tooltips
        self.setToolTip('Write the value for %s here ' % self._parameter.name)
        self._line_edit_input.setText(self._parameter.value)

        self.inner_input_layout.addWidget(self._line_edit_input)

    def get_parameter(self):
        """Obtain string parameter object from the current widget state.

        :returns: A StringParameter from the current state of widget
        """
        value = self._line_edit_input.text()
        if value.__class__.__name__ == 'QString':
            value = str(value)
        self._parameter.value = value
        return self._parameter

    def set_text(self, text):
        """Update the text of the widget

        :param text: The new text
        :type text: str
        """
        self._line_edit_input.setText(text)



# coding=utf-8
"""Tests for dict parameter."""

from unittest import TestCase

from parameters.parameter_exceptions import (
    CollectionLengthError,
    InvalidMinimumError,
    InvalidMaximumError)

from parameters.dict_parameter import DictParameter


good_dict = {'foo': True, 'bar': False, 'woo': False}
bad_dict = {'foo': True, 'bar': 'wawa', '': ''}


class TestListParameter(TestCase):

    def setUp(self):
        self.parameter = DictParameter()
        self.parameter.is_required = True
        self.parameter.element_type = bool

        self.parameter.minimum_item_count = 1
        self.parameter.maximum_item_count = 3
        self.parameter.value = good_dict

    def test_all(self):
        """General test."""

        self.assertEqual(good_dict, self.parameter.value)

        with self.assertRaises(TypeError):
            self.parameter.value = 'Test'

        with self.assertRaises(TypeError):
            self.parameter.value = 3.0

        with self.assertRaises(TypeError):
            self.parameter.value = 3

    def test_minimum_item_count(self):
        """Test minimum item count mutator."""
        with self.assertRaises(InvalidMinimumError):
            # Test what happens if the minimum is set to greater than max
            self.parameter.minimum_item_count = 4

        with self.assertRaises(CollectionLengthError):
            # and what happens if you try to load a list with less items
            # than the required minimum
            self.parameter.maximum_item_count = 8
            self.parameter.minimum_item_count = 4
            self.parameter.value = good_dict

    def test_maximum_item_count(self):
        """Test maximum item count mutator."""
        with self.assertRaises(InvalidMaximumError):
            # Test what happens if the maximum is set to less than min
            self.parameter.maximum_item_count = 0

        with self.assertRaises(CollectionLengthError):
            # and what happens if you try to load a list with more items than
            # required maximum
            self.parameter.minimum_item_count = 1
            self.parameter.maximum_item_count = 2
            self.parameter.value = good_dict

    def test_count(self):
        """Text count method."""
        self.parameter.value = good_dict
        self.assertEqual(len(good_dict), self.parameter.count())

    def test_element_type(self):
        """Test element type method."""
        self.parameter.element_type = int
        with self.assertRaises(TypeError):
            self.parameter.value = bad_dict

    def test_value(self):
        """Test value mutator."""
        self.parameter.value = good_dict
        self.assertEqual(good_dict, self.parameter.value)



# coding=utf-8
"""Parameter Container."""

from PyQt4.QtGui import (
    QWidget,
    QScrollArea,
    QVBoxLayout,
    QGridLayout,
    QSizePolicy,
    QColor,
    QLabel,
    QFrame,
    QHBoxLayout
)
from parameter_exceptions import InvalidValidationException

from parameters.qt_widgets.qt4_parameter_factory import Qt4ParameterFactory

__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'parameter_container'
__date__ = '8/22/14'
__copyright__ = 'ismail@kartoza.com'


class ParameterContainer(QWidget, object):
    """Container to hold Parameter Widgets."""

    def __init__(
            self,
            parameters=None,
            description_text='',
            extra_parameters=None,
            parent=None,
            vertical=True):
        """Constructor

        .. versionadded:: 2.2

        :param parameters: List of Parameter Widget
        :type parameters: list

        :param description_text: Text for description of the parameter
            container.
        :type description_text: str

        """
        QWidget.__init__(self, parent)
        # attributes
        if not parameters:
            self.parameters = []
        else:
            self.parameters = parameters
        self.description_text = description_text
        self.extra_parameters = extra_parameters
        self.parent = parent
        self.validators = []
        self.validators_kwargs = []

        # UI
        if vertical:
            self.vertical_layout = QVBoxLayout()
        else:
            self.vertical_layout = QHBoxLayout()
        self.widget = QWidget()
        self.description_label = QLabel()
        self.description_label.setWordWrap(True)
        self.scroll_area = QScrollArea()
        self.group_frame = QFrame()
        self.qt4_parameter_factory = Qt4ParameterFactory()
        self.main_layout = QGridLayout()

# NOTES(IS) : These functions are commented since the architecture is not
    #  ready yet.
    # def register_widget(self, parameter, parameter_widget):
    #     """Register new custom widget.
    #
    #     :param parameter:
    #     :type parameter: GenericParameter
    #
    #     :param parameter_widget:
    #     :type parameter_widget: GenericParameterWidget
    #     """
    #     self.qt4_parameter_factory.register_widget(
    # parameter, parameter_widget)
    #
    # def remove_widget(self, parameter):
    #     """Register new custom widget.
    #
    #     :param parameter:
    #     :type parameter: GenericParameter
    #     """
    #     if parameter.__name__ in self.dict_widget.keys():
    #         self.dict_widget.pop(parameter.__name__)

    def get_parameters(self, validate=True):
        """Return list of parameters from the current state of widget.

        :param validate: If true, run validator, else no.
        :type validate: bool

        :returns: List of parameter
        :rtype: list
        """
        if validate:
            validation_result = self.validate()
            if not validation_result['valid']:
                raise InvalidValidationException(validation_result['message'])

        parameter_widgets = self.get_parameter_widgets()

        parameters = []

        for widget_item in parameter_widgets:
            parameter_widget = widget_item.widget()

            parameter = parameter_widget.get_parameter()
            parameters.append(parameter)

        # returns based on the object type of self.parameters
        if isinstance(self.parameters, list):
            return parameters
        else:
            # just return single parameter
            return parameters[0]

    def get_parameter_widgets(self):
        """Return list of parameter widgets from the current state of widget.

        :returns: List of parameter widget
        :rtype: list
        """

        parameter_widgets = [self.vertical_layout.itemAt(i) for i in range(
            self.vertical_layout.count())]

        return parameter_widgets

    def setup_ui(self, must_scroll=True):
        """Setup the UI of this parameter container.
        """
        # Vertical layout to place the parameter widgets
        self.vertical_layout.setContentsMargins(0, 0, 0, 0)
        self.vertical_layout.setSpacing(0)

        # Widget to hold the vertical layout
        self.widget = QWidget()
        self.widget.setLayout(self.vertical_layout)

        # Label for description
        self.description_label.setText(self.description_text)

        self.group_frame.setLineWidth(0)
        self.group_frame.setFrameStyle(QFrame.NoFrame)
        vlayout = QVBoxLayout()
        vlayout.setContentsMargins(0, 0, 0, 0)
        vlayout.setSpacing(0)
        self.group_frame.setLayout(vlayout)

        if must_scroll:
            vlayout.addWidget(self.scroll_area)
            self.scroll_area.setWidgetResizable(True)
            self.scroll_area.setWidget(self.widget)
        else:
            vlayout.addWidget(self.widget)

        # Main layout of the container
        if self.description_text:
            self.main_layout.addWidget(self.description_label)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.main_layout)

        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.MinimumExpanding)

        if not isinstance(self.parameters, list):
            parameters = [self.parameters]
        else:
            parameters = self.parameters

        if len(parameters) == 0:
            self.set_empty_parameters()
            return

        self.main_layout.addWidget(self.group_frame)

        self.qt4_parameter_factory = Qt4ParameterFactory()
        if self.extra_parameters is not None:
            for extra_parameter in self.extra_parameters:
                if (type(extra_parameter) == tuple and
                        len(extra_parameter) == 2):
                    self.qt4_parameter_factory.register_widget(
                        extra_parameter[0], extra_parameter[1])

        color_odd = QColor(220, 220, 220)
        color_even = QColor(192, 192, 192)

        i = 0
        for parameter in parameters:
            parameter_widget = self.qt4_parameter_factory.get_widget(parameter)
            if i % 2:
                color = color_even
            else:
                color = color_odd
            i += 1
            parameter_widget.setAutoFillBackground(True)
            # palette = parameter_widget.palette()
            # palette.setColor(parameter_widget.backgroundRole(), color)
            # parameter_widget.setPalette(palette)
            self.vertical_layout.addWidget(parameter_widget)

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

    def set_description(self, description):
        """Set description of the parameter container.

        :param description: A new description fot the parameter container.
        :type description: str
        """
        self.description_text = description
        self.description_label.setText(self.description_text)

    def set_empty_parameters(self):
        """Update UI if there is no parameters in the container.
        """
        new_description = self.description_text
        new_description += '\n'
        new_description += 'But, currently there is no parameters available.'
        self.description_label.setText(new_description)

    def add_validator(self, validator, **kwargs):
        """Add validator for this parameter container.

        :param validator: validator function for this parameter container.
        :type validator: function
        """
        validator.parent = self
        self.validators.append(validator)
        if kwargs:
            self.validators_kwargs.append(kwargs)
        else:
            self.validators_kwargs.append({})

    def validate(self):
        """Validate of all rule for all parameter in this container.

        :return: True if all valid, False
        :rtype: dict
        """
        for i in range(len(self.validators)):
            validator = self.validators[i]
            validator_kwargs = self.validators_kwargs[i]
            validation_result = validator(self, **validator_kwargs)
            if not validation_result['valid']:
                return validation_result

        return {
            'valid': True,
            'message': ''
        }

    def get_parameter_by_guid(self, parameter_guid):
        """Return a parameter based on its uuid

        :param parameter_guid: The parameter uuid
        :type parameter_guid: str

        :returns: The parameter or None if not exist
        :rtype: GenericParameter, None
        """
        parameters = self.get_parameters(validate=False)
        for parameter in parameters:
            if parameter.guid == parameter_guid:
                return parameter
        return None

    def get_parameter_widget_by_guid(self, parameter_guid):
        """Return a parameter widget based on its uuid

        :param parameter_guid: The parameter uuid
        :type parameter_guid: str

        :returns: The parameter widget or None if not exist
        :rtype: GenericParameterWidget, None
        """
        parameter_widgets = self.get_parameter_widgets()
        for parameter_widget in parameter_widgets:
            if (parameter_widget.widget().get_parameter().guid ==
                    parameter_guid):
                return parameter_widget.widget()
        return None



# coding=utf-8
"""Tests for generic parameter."""

from unittest import TestCase

from parameters.generic_parameter import GenericParameter


class TestGenericParameter(TestCase):
    """Test generic parameter."""
    def test_guid(self):
        """Test mutator for guid."""
        guid = '1213231'
        parameter = GenericParameter()
        parameter.guid = guid
        self.assertEqual(guid, parameter.guid)
        parameter = GenericParameter(guid)
        self.assertEqual(guid, parameter.guid)

    def test_name(self):
        """Test mutator for name."""
        name = 'Foo'
        parameter = GenericParameter()
        parameter.name = name
        self.assertEqual(name, parameter.name)

    def test_expected_type(self):
        """Test mutator for expected type."""
        parameter_type = int
        parameter = GenericParameter()
        parameter.type = parameter_type
        self.assertEqual(parameter_type, parameter.type)

    def test_is_required(self):
        """Test mutator for is_required."""
        flag = True
        parameter = GenericParameter()
        parameter.is_required = flag
        self.assertEqual(flag, parameter.is_required)

    def test_help_text(self):
        """Test mutator for help text."""
        help_text = 'Foo'
        parameter = GenericParameter()
        parameter.help_text = help_text
        self.assertEqual(help_text, parameter.help_text)

    def test_set_description(self):
        """Test mutator for description."""
        help_description = 'Foo'
        parameter = GenericParameter()
        parameter.description = help_description
        self.assertEqual(help_description, parameter.description)

    def test_set_value(self):
        value = 'Foo'
        parameter = GenericParameter()
        parameter.type = str
        parameter.value = value
        self.assertEqual(value, parameter.value)



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'point_parameter_widget'
__date__ = '12/15/14'
__copyright__ = 'ismail@kartoza.com'

from PyQt4.QtGui import QSpinBox, QSizePolicy

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget


class PointParameterWidget(GenericParameterWidget):
    """Widget class for Integer parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A IntegerParameter object.
        :type parameter: IntegerParameter

        """
        # Size policy
        self._spin_box_size_policy = QSizePolicy(
            QSizePolicy.Fixed, QSizePolicy.Fixed)

        super(PointParameterWidget, self).__init__(parameter, parent)

        self._input_x = QSpinBox()
        self._input_x.setValue(self._parameter.value[0])
        tool_tip = 'X'
        self._input_x.setToolTip(tool_tip)
        self._input_x.setSizePolicy(self._spin_box_size_policy)

        self._input_y = QSpinBox()
        self._input_y.setValue(self._parameter.value[1])
        tool_tip = 'Y'
        self._input_y.setToolTip(tool_tip)
        self._input_y.setSizePolicy(self._spin_box_size_policy)

        self.inner_input_layout.addWidget(self._input_x)
        self.inner_input_layout.addWidget(self._input_y)

    def get_parameter(self):
        """Obtain boolean parameter object from the current widget state.

        :returns: A BooleanParameter from the current state of widget

        """
        self._parameter.value = (self._input_x.value(), self._input_y.value())
        return self._parameter



# coding=utf-8
"""Generic Parameter."""
import uuid


class GenericParameter(object):
    """A generic base class for all parameters."""

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str, None
        """
        self.guid = None
        if guid is None:
            self._guid = uuid.uuid4()
        else:
            self._guid = guid
        self._name = None
        self._expected_type = None
        self._required = None
        self._help_text = None
        self._description = None
        self._value = None
        # Defaults to _required
        self._is_required = True

    @property
    def guid(self):
        """Unique identifier property for a parameter.

        :returns: A globally unique identifier for the parameter.
        :rtype: str
        """
        return self._guid

    @guid.setter
    def guid(self, guid):
        """Assign a unique identifier to the parameter instance.

        :param guid: A globally unique identified (never translated).
        :type guid: str

        Perhaps it should set the _guid itself?
        """
        self._guid = guid

    @property
    def name(self):
        """Name property for the parameter.

        :returns: The name for this parameter.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Set the name for the parameter.

        :param name: The _name for the parameters. This will be used in the UI,
            and can be translated)
        :type name: str
        """
        self._name = name

    @property
    def expected_type(self):
        """Property for the expected type for this parameter.

        :returns: The expected type for this parameter.
        :rtype: str
        """
        return self._expected_type

    @expected_type.setter
    def expected_type(self, expected_type):
        """Define what type of input is _required.

        :param expected_type: ?????
        :type expected_type: ????

        """
        # TODO some validation here...
        # I think validation should be in set_value
        self._expected_type = expected_type

    @property
    def is_required(self):
        """Property indicating whether the parameter is required.

        :returns: bool
        """
        return self._is_required

    @is_required.setter
    def is_required(self, required):
        """Define if this is a required parameter or not.

        :param required: A _required to indicate if a parameter is _required.
        :type required: bool
        """
        self._is_required = required

    @property
    def description(self):
        """Property for the description of the parameter.

        :returns: A short description for the parameter.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Define the _description for this parameter.

        :param description: A short _description of the parameter
        :type description: str
        """
        self._description = description

    @property
    def help_text(self):
        """Property containing help text for the parameter.

        :returns: A long and detail help sentence for the parameter.
        :rtype: str
        """
        return self._help_text

    @help_text.setter
    def help_text(self, help_text):
        """Define the help _help_text for this parameter.

        :param help_text: A long and detail explanation of the parameter.
        :type help_text: str
        """
        self._help_text = help_text

    def _check_type(self, value):
        # Checking that the type of _value is the same as the expected _value
        # or same as one type in list of type
        message = (
            'The type of the _value [%s] does match with the expected '
            'type of the parameter [%s].' % (
                str(type(value)), str(self._expected_type)))

        if (type(self._expected_type) is type and type(value) is not
                self._expected_type):
            raise TypeError(message)
        if (type(self._expected_type) is list and type(value) not in
                self._expected_type):
            raise TypeError(message)

    @property
    def value(self):
        """Property for the parameter value.

        :returns: The parameter value. The return type depends on the
            subclass of GenericParameter.
        """
        return self._value

    @value.setter
    def value(self, value):
        """Define the current _value for the parameter.

        .. note:: Subclasses may want to overload this function to do parameter
            type specific checks.

        :param value: The _value of the parameter itself.
        :type value: str, bool, integer, float, list, dict

        :raises: TypeError
        """
        self._check_type(value)
        self._value = value

    def serialize(self):
        """Convert the parameter into a dictionary.

        :return: The parameter dictionary.
        :rtype: dict
        """
        # noinspection PyDictCreation
        pickle = {}
        pickle['guid'] = '%s' % self.guid
        pickle['name'] = self.name
        if type(self.expected_type) == list:
            pickle['expected_type'] = ['%s' % t for t in self.expected_type]
        else:
            pickle['expected_type'] = '%s' % self.expected_type
        pickle['is_required'] = self.is_required
        pickle['help_text'] = self.help_text
        pickle['description'] = self.description
        pickle['value'] = self.value
        return pickle



# coding=utf-8

__author__ = 'lucernae'
__project_name__ = 'parameters'
__filename__ = 'list_parameter_widget'
__date__ = '02/04/15'
__copyright__ = 'lana.pcfre@gmail.com'

from PyQt4.QtCore import Qt
from PyQt4.QtGui import (
    QListWidget, QAbstractItemView, QLineEdit, QVBoxLayout, QHBoxLayout, QPushButton, QListWidgetItem)

from parameters.qt_widgets.generic_parameter_widget import GenericParameterWidget
from parameters.input_list_parameter import InputListParameter


class InputListParameterWidget(GenericParameterWidget):
    """Widget class for List parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A ListParameter object.
        :type parameter: InputListParameter

        """
        super(InputListParameterWidget, self).__init__(parameter, parent)

        # value cache for self._parameter.value
        # copy the list so the original is unaffected
        self._value_cache = list(self._parameter.value)

        self.input = QListWidget()

        self.input.setSelectionMode(QAbstractItemView.SingleSelection)

        if self._parameter.maximum_item_count != \
                self._parameter.minimum_item_count:
            tool_tip = 'Select between %d and %d items' % (
                self._parameter.minimum_item_count,
                self._parameter.maximum_item_count)
        else:
            tool_tip = 'Select exactly %d items' % (
                       self._parameter.maximum_item_count)

        self.input.setToolTip(tool_tip)

        # arrange widget

        self.insert_item_input = QLineEdit()

        vbox_layout = QVBoxLayout()
        hbox_layout = QHBoxLayout()
        self.input_add_button = QPushButton('Add')
        self.input_remove_button = QPushButton('Remove')
        # arrange line edit, add button, remove button in horizontal layout
        hbox_layout.addWidget(self.insert_item_input)
        hbox_layout.addWidget(self.input_add_button)
        hbox_layout.addWidget(self.input_remove_button)
        # arrange vertical layout
        vbox_layout.addLayout(hbox_layout)
        vbox_layout.addWidget(self.input)
        self.inner_input_layout.addLayout(vbox_layout)

        # override self._input_layout arrangement to make the label at the top
        # reset the layout
        self.input_layout.setParent(None)
        self.help_layout.setParent(None)

        self.label.setParent(None)
        self.inner_input_layout.setParent(None)

        self.input_layout = QVBoxLayout()
        self.input_layout.setSpacing(0)

        # put element into layout
        self.input_layout.addWidget(self.label)
        self.input_layout.addLayout(self.inner_input_layout)

        self.main_layout.addLayout(self.input_layout)
        self.main_layout.addLayout(self.help_layout)

        # connect handler
        # noinspection PyUnresolvedReferences
        self.input_add_button.clicked.connect(self.on_add_button_click)
        # noinspection PyUnresolvedReferences
        self.input_remove_button.clicked.connect(self.on_remove_button_click)
        # noinspection PyUnresolvedReferences
        self.insert_item_input.returnPressed.connect(
            self.input_add_button.click)
        # noinspection PyUnresolvedReferences
        self.input.itemChanged.connect(self.on_row_changed)

        self.refresh_list()

        # init row add error handler
        self._add_row_error_handler = None

    @property
    def add_row_error_handler(self):
        """return error handler if user mistakenly add row of unexpected type
        :return: a function handler
        :rtype: () -> None
        """
        return self._add_row_error_handler

    @add_row_error_handler.setter
    def add_row_error_handler(self, value):
        """Set error handler to handle user mistakenly add row of unexpected
        type
        """
        self._add_row_error_handler = value

    def refresh_list(self):
        self.input.clear()
        if not self._parameter.ordering == InputListParameter.NotOrdered:
            self._value_cache.sort()
        if self._parameter.ordering == InputListParameter.DescendingOrder:
            self._value_cache.reverse()
        for opt in self._value_cache:
            item = QListWidgetItem()
            item.setText(str(opt))
            item.setFlags(item.flags() | Qt.ItemIsEditable)
            self.input.addItem(item)

    def on_add_button_click(self):
        try:
            value = self._parameter.element_type(
                self.insert_item_input.text())
            self._value_cache.append(value)
            self.refresh_list()
        except ValueError:
            err = self.raise_invalid_type_exception()
            if self.add_row_error_handler is not None:
                self.add_row_error_handler(err)
            else:
                raise err

    def on_remove_button_click(self):
        for opt in self.input.selectedItems():
            index = self.input.indexFromItem(opt)
            del self._value_cache[index.row()]
        self.refresh_list()

    def on_row_changed(self, item):
        try:
            index = self.input.indexFromItem(item).row()
            prev_value = self._value_cache[index]
            self._value_cache[index] = self._parameter.element_type(
                item.text())
            self.refresh_list()
        except ValueError:
            item.setText(str(prev_value))
            self.raise_invalid_type_exception()

    def raise_invalid_type_exception(self):
        message = 'Expecting element type of %s' % (
            self._parameter.element_type.__name__)
        err = ValueError(message)
        return err

    def get_parameter(self):
        """Obtain list parameter object from the current widget state.

        :returns: A ListParameter from the current state of widget

        """

        try:
            self._parameter.value = self._value_cache
        except ValueError:
            err = self.raise_invalid_type_exception()
            raise err

        return self._parameter



# coding=utf-8
"""Tests for select parameter."""

from unittest import TestCase

from parameters.parameter_exceptions import ValueNotAllowedException

from parameters.select_parameter import SelectParameter

__author__ = 'ismailsunni'
__project_name__ = 'parameters'
__filename__ = 'test_select_parameter'
__date__ = '05/10/2016'
__copyright__ = 'imajimatika@gmail.com'

selected = 'one'

options = ['one', 'two', 'three', 'four', 'five']


class TestSelectParameter(TestCase):
    """Test For Select Parameter."""

    def setUp(self):
        self.parameter = SelectParameter()
        self.parameter.is_required = True
        self.parameter.element_type = str

        self.parameter.options_list = options
        self.parameter.value = selected

    def test_set_value(self):
        self.parameter.value = selected
        self.assertEqual(selected, self.parameter.value)

    def test_not_allowed_value(self):
        with self.assertRaises(ValueNotAllowedException):
            self.parameter.value = 1



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'float_parameter_widget'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'

from PyQt4.QtGui import QSpinBox

from parameters.qt_widgets.numeric_parameter_widget import NumericParameterWidget


class IntegerParameterWidget(NumericParameterWidget):
    """Widget class for Integer parameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 2.2

        :param parameter: A IntegerParameter object.
        :type parameter: IntegerParameter

        """
        super(IntegerParameterWidget, self).__init__(parameter, parent)

        self._input = QSpinBox()
        self._input.setValue(self._parameter.value)
        self._input.setMinimum(self._parameter.minimum_allowed_value)
        self._input.setMaximum(self._parameter.maximum_allowed_value)
        tool_tip = 'Choose a number between %d and %d' % (
            self._parameter.minimum_allowed_value,
            self._parameter.maximum_allowed_value)
        self._input.setToolTip(tool_tip)

        self._input.setSizePolicy(self._spin_box_size_policy)

        self.inner_input_layout.addWidget(self._input)
        self.inner_input_layout.addWidget(self._unit_widget)



# coding=utf-8
"""Float Parameter."""
import sys

from numeric_parameter import NumericParameter


class FloatParameter(NumericParameter):
    """A subclass of generic parameter that accepts float only.

    .. note:: By default the min and max allowed values will be
        the platform specific largest and smallest float numbers.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(FloatParameter, self).__init__(guid)
        self.expected_type = [float, int]  # IS:because int is subset of float
        self._minimum_allowed_value = sys.float_info.min
        self._maximum_allowed_value = sys.float_info.max
        self._unit = ''
        # precision means the number of digit after comma that are considered
        self._precision = 2

    @property
    def precision(self):
        """Property for the precision for the parameter.

        :returns: The precision of the value of the parameter.
        :rtype: int
        """
        return self._precision

    @precision.setter
    def precision(self, precision):
        """Setter for the precision for the parameter.

        # precision means the number of digit after comma that are considered

        :param precision: The precision of the value of the parameter.
        :type precision: int

        """
        self._precision = precision

    def serialize(self):
        """Convert the parameter into a dictionary.

        :return: The parameter dictionary.
        :rtype: dict
        """
        pickle = super(FloatParameter, self).serialize()
        pickle['precision'] = self.precision
        return pickle



# coding=utf-8
"""Test class for boolean_parameter_widget."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'test_boolean_parameter_widget'
__date__ = '8/19/14'
__copyright__ = 'ismail@kartoza.com'

import unittest

from PyQt4.QtGui import QApplication

from parameters.qt_widgets.boolean_parameter_widget import BooleanParameterWidget
from parameters.boolean_parameter import BooleanParameter


class TestBooleanParameterWidget(unittest.TestCase):
    application = QApplication([])

    def test_init(self):
        parameter = BooleanParameter('1231231')
        parameter.name = 'Boolean'
        parameter.help_text = 'A boolean parameter'
        parameter.description = 'A test _description'
        parameter.is_required = True

        parameter.value = True

        widget = BooleanParameterWidget(parameter)

        expected_value = parameter.name
        real_value = widget.label.text()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        expected_value = parameter.value
        real_value = widget._check_box_input.isChecked()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

        # change value
        widget._check_box_input.setChecked(False)

        expected_value = False
        real_value = widget._check_box_input.isChecked()
        message = 'Expected %s get %s' % (expected_value, real_value)
        self.assertEqual(expected_value, real_value, message)

if __name__ == '__main__':
    unittest.main()



# coding=utf-8
"""Tests for string parameter."""

from unittest import TestCase

from parameters.string_parameter import StringParameter


class TestStringParameter(TestCase):
    def test_boolean(self):
        """Test a bool parameter works properly.

        ..versionadded:: 2.2

        """
        parameter = StringParameter('1231231')
        parameter.name = 'String parameter'
        parameter.help_text = 'A string parameter'
        parameter.description = 'A test description'
        parameter.is_required = True

        parameter.value = 'Yogyakarta'
        self.assertEqual('Yogyakarta', parameter.value)

        with self.assertRaises(TypeError):
            parameter.value = 1



# coding=utf-8
"""Tests for list parameter."""

from unittest import TestCase

from parameters.parameter_exceptions import (
    CollectionLengthError,
    InvalidMinimumError,
    InvalidMaximumError,
    ValueNotAllowedException)

from parameters.list_parameter import ListParameter


good_list = ['one', 'two', 'three']
bad_list = ['one', 2, 3.0]
int_list = [1, 2, 3]

not_allowed_list = ['one', 'two', 'six']

good_options = ['one', 'two', 'three', 'four', 'five']


class TestListParameter(TestCase):

    def setUp(self):
        self.parameter = ListParameter()
        self.parameter.is_required = True
        self.parameter.element_type = str

        self.parameter.minimum_item_count = 1
        self.parameter.maximum_item_count = 3
        self.parameter.options_list = good_options
        self.parameter.value = good_list

    def test_all(self):
        """General test."""

        self.assertEqual(good_list, self.parameter.value)

        with self.assertRaises(TypeError):
            # should raise, because it expects a list or dict of string
            self.parameter.value = 'Test'

        with self.assertRaises(TypeError):
            # should raise, because it expects a list or dict of string
            self.parameter.value = [3.0]

        with self.assertRaises(TypeError):
            # should raise, because it expects a list or dict of string
            self.parameter.value = [3]

    def test_set_minimum_item_count(self):
        with self.assertRaises(InvalidMinimumError):
            # Test what happens if the minimum is set to greater than max
            self.parameter.minimum_item_count = 4

        with self.assertRaises(CollectionLengthError):
            # and what happens if you try to load a list with less items
            # than the required minimum
            self.parameter.maximum_item_count = 8
            self.parameter.minimum_item_count = 4
            self.parameter.value = good_list

    def test_set_maximum_item_count(self):

        with self.assertRaises(InvalidMaximumError):
            # Test what happens if the maximum is set to less than min
            self.parameter.maximum_item_count = 0

        with self.assertRaises(CollectionLengthError):
            # and what happens if you try to load a list with less items
            # than the required minimum
            self.parameter.minimum_item_count = 1
            self.parameter.maximum_item_count = 2
            self.parameter.value = good_list

    def test_count(self):
        self.parameter.value = good_list
        self.assertEqual(len(good_list), self.parameter.count())

    def test_set_element_type(self):
        self.test_set_element_type = int
        with self.assertRaises(TypeError):
            self.parameter.value = bad_list

    def test_set_value(self):
        self.parameter.value = good_list
        self.assertEqual(good_list, self.parameter.value)

    def test_not_allowed_value(self):
        with self.assertRaises(ValueNotAllowedException):
            self.parameter.value = not_allowed_list



# coding=utf-8
"""Setup file for distutils / pypi."""
try:
    from ez_setup import use_setuptools
    use_setuptools()
except ImportError:
    pass

from setuptools import setup, find_packages


setup(
    name='inasafe-parameters',
    version='1.0.1',
    packages=find_packages(),
    license='GPL',
    author='InaSAFE Team',
    author_email='info@inasafe.org',
    url='http://inasafe.org/',
    description=('A generic package for capturing and validating user '
                 'parameters'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: GIS',
    ],
)



# coding=utf-8
"""Docstring for this file."""
__author__ = 'ismailsunni'
__project_name = 'parameters'
__filename = 'point_parameter'
__date__ = '12/15/14'
__copyright__ = 'ismail@kartoza.com'


from parameters.generic_parameter import GenericParameter


class PointParameter(GenericParameter):
    """Parameter that represent a point."""

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter.
            If none is specified one will be generated using python
            hash. This guid will be used when storing parameters in
            the registry.
        :type guid: str
        """
        super(PointParameter, self).__init__(guid)
        self.expected_type = tuple



# coding=utf-8
"""Boolean Parameter."""
from generic_parameter import GenericParameter


class BooleanParameter(GenericParameter):
    """A subclass of generic parameter that accepts boolean only.

    .. versionadded:: 2.2
    """

    def __init__(self, guid=None):
        """Constructor.

        :param guid: Optional unique identifier for this parameter. If none
            is specified one will be generated using python hash. This guid
            will be used when storing parameters in the registry.
        :type guid: str
        """
        super(BooleanParameter, self).__init__(guid)
        self.expected_type = bool



# coding=utf-8
"""Tests for boolean parameter."""
from unittest import TestCase

from parameters.boolean_parameter import BooleanParameter


class TestBooleanParameter(TestCase):

    def test_boolean(self):
        """Test a bool parameter works properly.

        .. versionadded:: 2.2

        """
        parameter = BooleanParameter('1231231')
        parameter.name = 'Boolean'
        parameter.help_text = 'A boolean parameter'
        parameter.description = 'A test _description'
        parameter.is_required = True

        parameter.value = True
        self.assertEqual(True, parameter.value)

        with self.assertRaises(TypeError):
            parameter.value = 'Test'



# coding=utf-8
"""Docstring for this file."""

__author__ = 'Jannes Engelbrecht jannes@kartoza.com'
__project_name = 'parameters'
__filename = 'dict_parameter_widget'
__date__ = '12/11/14'
__copyright__ = 'kartoza.com'

# This import is to enable SIP API V2
# noinspection PyPackageRequirements
import logging

from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QTreeWidget, QFont, QTreeWidgetItem, QVBoxLayout)

from parameters.qt_widgets.generic_parameter_widget import (
    GenericParameterWidget)

LOGGER = logging.getLogger('InaSAFE')


# pylint: disable=super-on-old-class
class DictParameterWidget(GenericParameterWidget):
    """Widget class for DictParameter."""
    def __init__(self, parameter, parent=None):
        """Constructor

        .. versionadded:: 3.1

        :param parameter: A DictParameter object.
        :type parameter: DictParameter

        """
        # pylint: disable=E1002
        super(DictParameterWidget, self).__init__(parameter, parent)
        # pylint: enable=E1002

        self.input = QTreeWidget()

        # generate tree model
        widget_items = self.generate_tree_model(self._parameter.value)
        self.input.addTopLevelItems(widget_items)
        # set header
        self.input.headerItem().setText(0, 'Keys')
        self.input.headerItem().setText(1, 'Values')

        self.inner_input_layout.addWidget(self.input)

        # override self._input_layout arrangement to make the label at the top
        # reset the layout
        self.input_layout.setParent(None)
        self.help_layout.setParent(None)

        self.label.setParent(None)
        self.inner_input_layout.setParent(None)

        self.input_layout = QVBoxLayout()
        self.input_layout.setSpacing(0)

        # put element into layout
        self.input_layout.addWidget(self.label)
        self.input_layout.addLayout(self.inner_input_layout)

        self.main_layout.addLayout(self.input_layout)
        self.main_layout.addLayout(self.help_layout)

    def generate_tree_model(self, data_dict):
        """Generate a tree model for specified dictionary

        :param data_dict: A dictionary
        :type data_dict: dict
        :return: list of QTreeWidgetItem
        :rtype list:
        """
        widget_items = []
        font = QFont()
        font.setBold(True)
        for key in data_dict.keys():
            entry = data_dict[key]
            key_item = QTreeWidgetItem()
            key_item.setText(0, str(key))
            key_item.setFont(0, font)
            if isinstance(entry, dict):
                items = self.generate_tree_model(entry)
                key_item.addChildren(items)
            else:
                # value_item = QTreeWidgetItem()
                key_item.setText(1, str(entry))
                key_item.setFlags(key_item.flags() | Qt.ItemIsEditable)
                # key_item.addChild(key_item)
            widget_items.append(key_item)

        return widget_items

    def extract_dict(self, widget_items):
        """Extract dictionary key and values from QTreeWidgetItems

        :param widget_items: List of QTreeWidgetItems
        :type widget_items: list
        :return: hierarchical dictionary extracted from widget_items
        :rtype dict:
        """
        data_dict = {}
        element_type = self._parameter.element_type
        if element_type == object:
            def object_cast(obj):
                return obj
            element_type = object_cast
        for key_item in widget_items:
            key = str(key_item.text(0))
            value = None
            if key_item.childCount() == 0:
                # value_item = key_item.child(0)
                value = element_type(key_item.text(1))
            elif key_item.childCount() > 1:
                value_items = [key_item.child(i)
                               for i in range(key_item.childCount())]
                value = self.extract_dict(value_items)
            data_dict[key] = value
        return data_dict

    def get_parameter(self):
        """Obtain the parameter object from the current widget state.

        :returns: A DictParameter from the current state of widget

        """
        root_widget_item = self.input.invisibleRootItem()
        widget_items = [root_widget_item.child(i)
                        for i in range(root_widget_item.childCount())]
        data_dict = self.extract_dict(widget_items)
        self._parameter.value = data_dict
        return self._parameter

# pylint: enable=super-on-old-class
